# screenviz.__main__

import argparse

from screenviz.compare import CompareScreens
from screenviz.gene import VisualizeGenes
from screenviz.idea import RunIDEA
from screenviz.qc import quality_control_app_entry
from screenviz.results import results_app_entry
from screenviz.sgrna import VisualizeSGRNAs


def geneviz_parser(subparser):
    # create the parser for the "geneviz" command
    parser_geneviz = subparser.add_parser(
        "gene", help="Visualize the volcano plot of the gene enrichment analysis"
    )
    parser_geneviz.add_argument("-i", "--input", help="Input file", required=True)
    parser_geneviz.add_argument(
        "-o",
        "--output",
        help="Output file (default = 'gene_volcano.html')",
        required=False,
        default="gene_volcano.html",
    )
    parser_geneviz.add_argument(
        "-c",
        "--config",
        help="Config file generated by `crispr_screen`",
        required=False,
    )
    parser_geneviz.add_argument(
        "-g",
        "--gene_column",
        help="Column name of gene names (default = 'gene')",
        required=False,
        default="gene",
    )
    parser_geneviz.add_argument(
        "-f",
        "--fc_column",
        help="Column name of fold change values (default = 'log_fold_change')",
        required=False,
        default="log_fold_change",
    )
    parser_geneviz.add_argument(
        "-p",
        "--pval_column",
        help="Column name of score column (default = 'pvalue')",
        required=False,
        default="pvalue",
    )
    parser_geneviz.add_argument(
        "-t",
        "--threshold_column",
        help="Column name of threshold column (default = 'fdr')",
        required=False,
        default="fdr",
    )
    parser_geneviz.add_argument(
        "-th",
        "--threshold",
        type=float,
        help="Threshold value (default = 0.1)",
        required=False,
        default=0.1,
    )


def sgrnaviz_parser(subparser):
    # create the parser for the "sgrnaviz" command
    parser_sgrnaviz = subparser.add_parser(
        "sgrna", help="Visualize the volcano plot of the sgRNA enrichment analysis"
    )
    parser_sgrnaviz.add_argument("-i", "--input", help="Input file", required=True)
    parser_sgrnaviz.add_argument(
        "-o",
        "--output",
        help="Output file (default = 'sgrna_volcano.html')",
        required=False,
        default="sgrna_volcano.html",
    )
    parser_sgrnaviz.add_argument(
        "-s",
        "--sgrna_column",
        help="Column name of sgRNA names (default = 'sgrna')",
        required=False,
        default="sgrna",
    )
    parser_sgrnaviz.add_argument(
        "-g",
        "--gene_column",
        help="Column name of gene names (default = 'gene')",
        required=False,
        default="gene",
    )
    parser_sgrnaviz.add_argument(
        "-f",
        "--fc_column",
        help="Column name of fold change values (default = 'log2_fold_change')",
        required=False,
        default="log2_fold_change",
    )
    parser_sgrnaviz.add_argument(
        "-p",
        "--pval_column",
        help="Column name of score column (default = 'pvalue')",
        required=False,
        default="pvalue_twosided",
    )
    parser_sgrnaviz.add_argument(
        "-t",
        "--threshold_column",
        help="Column name of threshold column (default = 'fdr')",
        required=False,
        default="fdr",
    )
    parser_sgrnaviz.add_argument(
        "-th",
        "--threshold",
        type=float,
        help="Threshold value (default = 0.1)",
        required=False,
        default=0.1,
    )


def compare_parser(subparser):
    parser_compare_gene = subparser.add_parser(
        "compare",
        help="Compare the gene enrichments between two analyses of the same screen",
    )
    parser_compare_gene.add_argument(
        "-i", "--screen_a", help="Input file to use as the first screen", required=True
    )
    parser_compare_gene.add_argument(
        "-I", "--screen_b", help="Input file to use as the second screen", required=True
    )
    parser_compare_gene.add_argument(
        "-x",
        "--variable_column_a",
        help="Column name to plot for the first screen",
        required=False,
        default="pvalue",
    )
    parser_compare_gene.add_argument(
        "-X",
        "--variable_column_b",
        help="Column name to plot for the second screen",
        required=False,
        default="pvalue",
    )
    parser_compare_gene.add_argument(
        "-t",
        "--threshold_column_a",
        help="Column name to use as the threshold for the first screen",
        required=False,
        default="fdr",
    )
    parser_compare_gene.add_argument(
        "-T",
        "--threshold_column_b",
        help="Column name to use as the threshold for the second screen",
        required=False,
        default="fdr",
    )
    parser_compare_gene.add_argument(
        "-th",
        "--threshold",
        type=float,
        help="Threshold value (default = 0.1)",
        required=False,
        default=0.1,
    )
    parser_compare_gene.add_argument(
        "-m",
        "--merge_column_a",
        help="Column to merge on the first screen",
        required=False,
        default="gene",
    )
    parser_compare_gene.add_argument(
        "-M",
        "--merge_column_b",
        help="Column to merge on the second screen",
        required=False,
        default="gene",
    )
    parser_compare_gene.add_argument(
        "-n",
        "--no_log_transform_a",
        help="Do not log transform the first screen",
        required=False,
        action="store_false",
    )
    parser_compare_gene.add_argument(
        "-N",
        "--no_log_transform_b",
        help="Do not log transform the second screen",
        required=False,
        action="store_false",
    )


def idea_parser(subparser):
    # create the parser for the "geneviz" command
    parser_idea = subparser.add_parser(
        "idea",
        help="Perform Gene Set Enrichment using Enrichr and visualize as an IDEA plot",
    )
    parser_idea.add_argument("-i", "--input", help="Input file", required=True)
    parser_idea.add_argument(
        "-o",
        "--output",
        help="Output file (default = '<prefix>.<geneset>.html')",
        required=False,
        default="network",
    )
    parser_idea.add_argument(
        "-s",
        "--geneset",
        help="Gene set to perform enrichment against (BP, MF, CC, or any name in Enrichr)",
        required=False,
        default="BP",
    )
    parser_idea.add_argument(
        "-g",
        "--gene_column",
        help="Column name of gene names (default = 'gene')",
        required=False,
        default="gene",
    )
    parser_idea.add_argument(
        "-f",
        "--fc_column",
        help="Column name of fold change values (default = 'log_fold_change')",
        required=False,
        default="log_fold_change",
    )
    parser_idea.add_argument(
        "-p",
        "--pval_column",
        help="Column name of score column (default = 'fdr')",
        required=False,
        default="fdr",
    )
    parser_idea.add_argument(
        "-t",
        "--threshold_column",
        help="Column name of threshold column (default = 'fdr')",
        required=False,
        default="fdr",
    )
    parser_idea.add_argument(
        "-th",
        "--threshold",
        type=float,
        help="Threshold value (default = 0.1) to use for differentially expressed genes",
        required=False,
        default=0.1,
    )
    parser_idea.add_argument(
        "-tth",
        "--term_threshold",
        type=float,
        help="Threshold value (default = 0.1) to use for enriched terms",
        required=False,
        default=0.1,
    )
    parser_idea.add_argument(
        "--sided",
        type=str,
        help="One-sided enrichment, choose either 'up' or 'down'",
        required=False,
    )
    parser_idea.add_argument(
        "--top",
        type=int,
        help="Number of top terms to show (default = 30)",
        required=False,
        default=30,
    )
    parser_idea.add_argument(
        "--term_palette",
        type=str,
        help="Color palette for term nodes",
        required=False,
        default="Greens",
    )
    parser_idea.add_argument(
        "--gene_palette",
        type=str,
        help="Color palette for gene nodes",
        required=False,
    )
    parser_idea.add_argument(
        "--up_color",
        type=str,
        help="Color palette for up-regulated genes",
        required=False,
    )
    parser_idea.add_argument(
        "--down_color",
        type=str,
        help="Color palette for down-regulated genes",
        required=False,
    )


def quality_control_parser(subparser):
    parser_quality_control = subparser.add_parser(
        "qc",
        help="Visualize quality control metrics interactively on the input data",
    )
    parser_quality_control.add_argument(
        "-i",
        "--input",
        help="Input file (this should be a count-matrix from the output of sgcount)",
        required=True,
    )
    parser_quality_control.add_argument(
        "-p",
        "--port",
        help="Port number to run the visualization on (default = 8050)",
        required=False,
        default=8050,
    )
    parser_quality_control.add_argument(
        "-s",
        "--guide-column",
        help="Column name of sgRNA names (default = 'Guide')",
        required=False,
        default="Guide",
    )
    parser_quality_control.add_argument(
        "-g",
        "--gene-column",
        help="Column name of gene names (default = 'Gene')",
        required=False,
        default="Gene",
    )


def results_parser(subparser):
    parser_results = subparser.add_parser(
        "results", help="Visualize CRISPR screen results interactively"
    )
    parser_results.add_argument(
        "-s", "--sgrna-file", help="Input file for sgRNA-level results", required=True
    )
    parser_results.add_argument(
        "-g", "--gene-file", help="Input file for gene-level results", required=True
    )
    parser_results.add_argument(
        "-p",
        "--port",
        help="Port number to run the visualization on (default = 8050)",
        type=int,
        default=8050,
    )


def get_args() -> argparse.Namespace:
    parser = argparse.ArgumentParser()
    subparser = parser.add_subparsers(dest="subcommand", required=True)
    geneviz_parser(subparser)
    sgrnaviz_parser(subparser)
    compare_parser(subparser)
    idea_parser(subparser)
    quality_control_parser(subparser)
    results_parser(subparser)
    return parser.parse_args()


def main_cli():
    args = get_args()
    if args.subcommand == "gene":
        vg = VisualizeGenes(
            filename=args.input,
            config=args.config,
            gene_column=args.gene_column,
            fc_column=args.fc_column,
            pval_column=args.pval_column,
            threshold_column=args.threshold_column,
            threshold=args.threshold,
        )
        vg.plot_volcano(output=args.output)
    elif args.subcommand == "sgrna":
        sg = VisualizeSGRNAs(
            filename=args.input,
            sgrna_column=args.sgrna_column,
            gene_column=args.gene_column,
            fc_column=args.fc_column,
            pval_column=args.pval_column,
            threshold_column=args.threshold_column,
            threshold=args.threshold,
        )
        sg.plot_volcano(output=args.output)
    elif args.subcommand == "compare":
        cs = CompareScreens(
            filename_a=args.screen_a,
            filename_b=args.screen_b,
            variable_column_a=args.variable_column_a,
            variable_column_b=args.variable_column_b,
            threshold_column_a=args.threshold_column_a,
            threshold_column_b=args.threshold_column_b,
            threshold=args.threshold,
            merge_column_a=args.merge_column_a,
            merge_column_b=args.merge_column_b,
            log_transform_a=~args.no_log_transform_a,
            log_transform_b=~args.no_log_transform_b,
        )
        cs.plot_volcano()
    elif args.subcommand == "idea":
        RunIDEA(
            filename=args.input,
            geneset=args.geneset,
            output=args.output,
            gene_column=args.gene_column,
            fc_column=args.fc_column,
            pval_column=args.pval_column,
            threshold_column=args.threshold_column,
            threshold=args.threshold,
            sided=args.sided,
            top=args.top,
            gene_palette=args.gene_palette,
            term_palette=args.term_palette,
            up_color=args.up_color,
            down_color=args.down_color,
            term_threshold=args.term_threshold,
        )
    elif args.subcommand == "qc":
        quality_control_app_entry(
            filename=args.input,
            port=args.port,
            guide_column=args.guide_column,
            gene_column=args.gene_column,
        )
    elif args.subcommand == "results":
        results_app_entry(
            sgrna_file=args.sgrna_file,
            gene_file=args.gene_file,
            port=args.port,
        )


if __name__ == "__main__":
    main_cli()
